#include <Carbon/Carbon.h>

static void CreateContentView(WindowRef);
static void InstallHandlers(WindowRef, HIViewRef);
static OSStatus DrawEventHandler(EventHandlerCallRef handler, EventRef event, void *userData);

#define check_err(err, ...) do { if (err != noErr) {fprintf(stderr, __VA_ARGS__);goto error;}} while (0)

#define rdebug(s) fprintf(stderr, s)

extern WindowGroupRef qd_nontopmost_group;

void* QUARTZDRV_CreateNewWindow(const char *const title, int x, int y,
								int width, int height) {
  OSStatus err;
  WindowRef window;
  WindowAttributes attrs;
  Rect bounds;
  CFStringRef windowTitle;
  
  attrs = kWindowStandardDocumentAttributes | kWindowStandardHandlerAttribute
	| kWindowCompositingAttribute;
  CARBON_SetRect(&bounds, x, y, 460, 380);
  err = CreateNewWindow(kDocumentWindowClass, attrs, &bounds, &window);
  check_err(err, "Can't create window %s\n", title);

  // Is this the right encoding?
  //windowTitle = CFStringCreateWithCString(NULL, title, kCFStringEncodingASCII);
  //SetWindowTitleWithCFString(window, windowTitle);

  //CFRelease(windowTitle);
  
  SetWindowGroup(window, qd_nontopmost_group);

  CreateContentView(window);

 error:
  return window;
  
}

static void CreateContentView(WindowRef window) {
  OSStatus err;
  HIRect bounds;
  const HILayoutInfo layout = {
	kHILayoutInfoVersionZero,
	{ { NULL, kHILayoutBindNone, 0.0 }, { NULL, kHILayoutBindNone, 0.0 }, 
	  { NULL, kHILayoutBindNone, 0.0 }, { NULL, kHILayoutBindNone, 0.0 }, 
	},
	{ { NULL, kHILayoutScaleAbsolute, 1.0 }, { NULL, kHILayoutScaleAbsolute, 1.0 } },
	{ { NULL, kHILayoutPositionMin, 0.0}, { NULL, kHILayoutPositionMin, 0.0 } }
  };

  HIViewRef parent, view;

  err = HIViewFindByID(HIViewGetRoot(window), kHIViewWindowContentID, &parent);
  check_err(err, "Can't find parent view\n");
  err = HIObjectCreate(kHIViewClassID, NULL, &view);
  check_err(err, "Can't create content view\n");

  err = HIViewAddSubview(parent, view);
  check_err(err, "Can't add view\n");

  err = HIViewGetBounds(parent, &bounds);
  check_err(err, "Can't get parent's bounds\n");

  err = HIViewSetFrame(view, &bounds);
  check_err(err, "Can't set view frame\n");

  err = HIViewSetLayoutInfo(view, &layout);
  check_err(err, "Can't set layout info\n");

  err = HIViewSetVisible(view, true);
  check_err(err, "Can't make view visible\n");

  InstallHandlers(window, view);
 error:
  return;
}

static void InstallHandlers(WindowRef window, HIViewRef view) {
  EventTypeSpec type;
  EventHandlerUPP handler;

  // add handlers for window, mouse and keyboard events

  // draw handler
  handler = NewEventHandlerUPP(DrawEventHandler);
  type.eventClass = kEventClassControl;
  type.eventKind = kEventControlDraw;
  InstallControlEventHandler(view, handler, 1, &type, view, NULL);
}

static OSStatus DrawEventHandler(EventHandlerCallRef handler, EventRef event, void *userData) {
  // send message to windows
  return noErr;
}
