/*
 * QUARTZ driver definitions
 *
 * Copyright 1998 Patrik Stridvall
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __WINE_QUARTZDRV_H
#define __WINE_QUARTZDRV_H

#ifndef __WINE_CONFIG_H
# error You must include config.h to use this header
#endif

#undef ERR
#if defined(HAVE_LIBCURSES) || defined(HAVE_LIBNCURSES)
#ifdef HAVE_NCURSES_H
# include <ncurses.h>
#elif defined(HAVE_CURSES_H)
# include <curses.h>
#endif
#endif
#include <stdarg.h>

#include "windef.h"
#include "winbase.h"
#include "wingdi.h"
#include "winuser.h"

struct tagBITMAPOBJ;

#if defined(HAVE_LIBCURSES) || defined(HAVE_LIBNCURSES)
#define WINE_CURSES
#endif

/**************************************************************************
 * QUARTZ GDI driver
 */

#ifndef WINE_CURSES
typedef struct { int dummy; } WINDOW;
#endif

typedef struct {
  HDC hdc;
  POINT org;
  WINDOW *window;
  int cellWidth;
  int cellHeight;
} QUARTZDRV_PDEVICE;

extern BOOL QUARTZDRV_GDI_Initialize(void);

/* QUARTZ GDI bitmap driver */

extern HBITMAP QUARTZDRV_BITMAP_CreateDIBSection(QUARTZDRV_PDEVICE *physDev, BITMAPINFO *bmi, UINT usage, LPVOID *bits, HANDLE section, DWORD offset);
extern void QUARTZDRV_BITMAP_DeleteDIBSection(struct tagBITMAPOBJ *bmp);

extern BOOL QUARTZDRV_DC_Arc(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom, INT xstart, INT ystart, INT xend, INT yend);
extern LONG QUARTZDRV_DC_BitmapBits(HBITMAP hbitmap, void *bits, LONG count, WORD flags);
extern BOOL QUARTZDRV_DC_DeleteDC(QUARTZDRV_PDEVICE *physDev);
extern BOOL QUARTZDRV_DC_BitBlt(QUARTZDRV_PDEVICE *physDevDst, INT xDst, INT yDst, INT width, INT height, QUARTZDRV_PDEVICE *physDevSrc, INT xSrc, INT ySrc, DWORD rop);
extern BOOL QUARTZDRV_DC_Chord(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom, INT xstart, INT ystart, INT xend, INT yend);
extern BOOL QUARTZDRV_DC_Ellipse(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom);
extern BOOL QUARTZDRV_DC_ExtFloodFill(QUARTZDRV_PDEVICE *physDev, INT x, INT y, COLORREF color, UINT fillType);
extern BOOL QUARTZDRV_DC_ExtTextOut(QUARTZDRV_PDEVICE *physDev, INT x, INT y, UINT flags, const RECT *lpRect, LPCWSTR str, UINT count, const INT *lpDx);
extern BOOL QUARTZDRV_DC_GetCharWidth(QUARTZDRV_PDEVICE *physDev, UINT firstChar, UINT lastChar, LPINT buffer);
extern COLORREF QUARTZDRV_DC_GetPixel(QUARTZDRV_PDEVICE *physDev, INT x, INT y);

extern BOOL QUARTZDRV_DC_GetTextExtentPoint(QUARTZDRV_PDEVICE *physDev, LPCWSTR str, INT count, LPSIZE size);
extern BOOL QUARTZDRV_DC_GetTextMetrics(QUARTZDRV_PDEVICE *physDev, TEXTMETRICW *metrics);
extern BOOL QUARTZDRV_DC_LineTo(QUARTZDRV_PDEVICE *physDev, INT x, INT y);
extern BOOL QUARTZDRV_DC_PaintRgn(QUARTZDRV_PDEVICE *physDev, HRGN hrgn);
extern BOOL QUARTZDRV_DC_PatBlt(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT width, INT height, DWORD rop);
extern BOOL QUARTZDRV_DC_Pie(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom, INT xstart, INT ystart, INT xend, INT yend);
extern BOOL QUARTZDRV_DC_Polygon(QUARTZDRV_PDEVICE *physDev, const POINT* pt, INT count);
extern BOOL QUARTZDRV_DC_Polyline(QUARTZDRV_PDEVICE *physDev, const POINT* pt, INT count);
extern BOOL QUARTZDRV_DC_PolyPolygon(QUARTZDRV_PDEVICE *physDev, const POINT* pt, const INT* counts, UINT polygons);
extern BOOL QUARTZDRV_DC_PolyPolyline(QUARTZDRV_PDEVICE *physDev, const POINT* pt, const DWORD* counts, DWORD polylines);
extern BOOL QUARTZDRV_DC_Rectangle(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom);
extern BOOL QUARTZDRV_DC_RoundRect(QUARTZDRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom, INT ell_width, INT ell_height);
extern COLORREF QUARTZDRV_DC_SetPixel(QUARTZDRV_PDEVICE *physDev, INT x, INT y, COLORREF color);
extern BOOL QUARTZDRV_DC_StretchBlt(QUARTZDRV_PDEVICE *physDevDst, INT xDst, INT yDst, INT widthDst, INT heightDst, QUARTZDRV_PDEVICE *physDevSrc, INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, DWORD rop);
INT QUARTZDRV_DC_SetDIBitsToDevice(QUARTZDRV_PDEVICE *physDev, INT xDest, INT yDest, DWORD cx, DWORD cy, INT xSrc, INT ySrc, UINT startscan, UINT lines, LPCVOID bits, const BITMAPINFO *info, UINT coloruse);

/* QUARTZ GDI palette driver */

extern BOOL QUARTZDRV_PALETTE_Initialize(void);

/**************************************************************************
 * QUARTZ USER driver
 */

#endif /* !defined(__WINE_QUARTZDRV_H) */
