#include <Carbon/Carbon.h>

// Put this in a separate file because Carbon.h conflicts with windows includes.

// Handle quit message.
WindowGroupRef qd_topmost_group, qd_nontopmost_group;

static OSStatus QuitHandler (EventHandlerCallRef myHandler, EventRef event, void *userData) {
    OSStatus status = noErr;
	QUARTZDRV_QuitWindowsApp();
    return status;
}

static OSStatus QuitMenuHandler (EventHandlerCallRef myHandler, EventRef event, void *userData) {
    OSStatus err = eventNotHandledErr;
    HICommand commandStruct;
    
    err = GetEventParameter (event, kEventParamDirectObject, typeHICommand, NULL, sizeof(HICommand), NULL, &commandStruct);
    require_noerr(err, error);
    
    if (commandStruct.commandID == kHICommandQuit) {
        EventRef e;
    
        err = CreateEvent(NULL, kEventClassApplication, kEventAppQuit, 0, kEventAttributeNone, &e);
        require_noerr(err, error);
    
        // should this be the application target?
        err = SendEventToEventTarget(e, GetEventDispatcherTarget());
        require_noerr(err, error);
        
        HiliteMenu(0);
        err = noErr;
    }
    
error:
    return err;
}

/***********************************************************************
 *           QUARTZDRV process initialisation routine
 */
void QUARTZDRV_process_attach(void)
{
	EventTypeSpec EventType;
	EventHandlerUPP handler;

	handler = NewEventHandlerUPP(QuitHandler);
	EventType.eventClass = kEventClassApplication;
	EventType.eventKind = kEventAppQuit;
	InstallApplicationEventHandler(handler, 1, &EventType, NULL, NULL);

	handler = NewEventHandlerUPP(QuitMenuHandler);
	EventType.eventClass = kEventClassCommand;
	EventType.eventKind = kEventCommandProcess;
	InstallApplicationEventHandler(handler, 1, &EventType, NULL, NULL);	

	// Create window groups
	CreateWindowGroup(kWindowGroupAttrSelectAsLayer,
					  &qd_topmost_group);
	SetWindowGroupLevel(qd_topmost_group,
						CGWindowLevelForKey(kCGFloatingWindowLevelKey));
	CreateWindowGroup(kWindowGroupAttrSelectAsLayer,
					  &qd_nontopmost_group);
	SetWindowGroupLevel(qd_nontopmost_group,
						CGWindowLevelForKey(kCGNormalWindowLevelKey));

    QUARTZDRV_GDI_Initialize();
}


/***********************************************************************
 *           QUARTZDRV process termination routine
 */
 void QUARTZDRV_process_detach(void)
{
  ReleaseWindowGroup(qd_nontopmost_group);
  ReleaseWindowGroup(qd_topmost_group);
}

char QUARTZDRV_GetEvent(EventRef *event) {
  WindowRef window;
  if (ReceiveNextEvent(0, NULL, kEventDurationNoWait, true, event) == noErr) {
	if (GetEventClass(*event) == kEventClassMouse 
		&& GetEventKind(*event) == kEventMouseDown) {
	  Point p;
	  GetEventParameter(*event, kEventParamMouseLocation, typeQDPoint, NULL,
						sizeof(Point), NULL, &p);
	  if (FindWindow(p, &window) == inMenuBar) {
		MenuSelect(p);
		goto finish;
	  }
	}
	SendEventToEventTarget(*event, GetEventDispatcherTarget());
  finish:
	ReleaseEvent(*event);
	return 1;
  }
  return 0;
}
